<?php

namespace Teknisa\Model\Teknisa\Repositories;

use Doctrine\DBAL\Connection;

class OmProdutoRepositorio extends \Doctrine\ORM\EntityRepository {

    public function findProductRepositories($conditions, $nrProdutoId){
        $query = <<<SQL
		SELECT *
		  FROM (
    		SELECT OPR.ID, OPR.NRPRODUTO_ID, OPR.NREXTERNAL_REPOSITORY_ID,
    		       OPR.IDATIVO, OPR.DSDIRRELMIGBD,
                   REP.URL, REP.PATH, REP.NAME AS EXTERNAL_REPOSITORY
              FROM OM_PRODUTO_REPOSITORIO OPR
              JOIN REPOSITORY REP ON REP.ID = OPR.NREXTERNAL_REPOSITORY_ID
             WHERE OPR.NRPRODUTO_ID = :NRPRODUTO_ID
               AND REP.STATUS = 'A')
		WHERE 1=1
		      $conditions
SQL;
        $param = array( 'NRPRODUTO_ID' => $nrProdutoId );
        $productRepositories = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($productRepositories as &$productRepository){
            $productRepository['ID'] = intval($productRepository['ID']);
        }
        return $productRepositories;
    }

    public function findNotInsertedProductRepositories($conditions, $externalProductId, $nrProdutoId){
        $query = <<<SQL
		SELECT *
		  FROM (
    		SELECT REP.ID, REP.URL, REP.PATH, REP.NAME, REP.PRODUCT
              FROM REPOSITORY REP
             WHERE REP.PRODUCT = :EXTERNAL_PRODUCT
               AND REP.STATUS = 'A'
               AND REP.ID NOT IN (SELECT REP.ID
                                    FROM OM_PRODUTO_REPOSITORIO OPR
                                    JOIN REPOSITORY REP ON REP.ID = OPR.NREXTERNAL_REPOSITORY_ID
                                   WHERE OPR.NRPRODUTO_ID = :NRPRODUTO_ID
                                     AND REP.STATUS = 'A'))
		WHERE 1=1
		      $conditions
SQL;
        $param = array(
            'NRPRODUTO_ID' => $nrProdutoId,
            'EXTERNAL_PRODUCT' => $externalProductId
        );

        $productRepositories = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($productRepositories as &$productRepository){
            $productRepository['ID'] = intval($productRepository['ID']);
        }
        return $productRepositories;
    }

    /**
     * @param $productIds
     * @param $deployIds
     * @return array
     */
    public function getMigrationRepositoriesInfoWithModules($productIds, $deployIds) {
        $query = "SELECT RPT.NAME, PRP.ID,
                         REPLACE (
                             REPLACE (
                                RPT.URL, 
                                'git@gitlab.teknisa.com:teknisa/entities.git',  'git@192.168.122.56:teknisa/entities.git'
                             ), 
                             'git@gitlab.teknisa.com:zeedhi/devops-entities.git',  'git@192.168.122.56:zeedhi/devops_entities.git'
                         ) AS URL,
                         PRP.DSDIRRELMIGBD AS MIG_RELATIVE_PATH,
                         DRP.INITIAL_MIGRATION_NUMBER,
                         REGEXP_REPLACE(PATH, (SELECT REP.PATH
                                                 FROM REPOSITORY REP
                                                WHERE REP.PRODUCT = RPT.PRODUCT
                                                  AND REP.STATUS = 'A'
                                                  AND REP.IS_MASTER = 'Y'), '', 1, 1) AS PATH
                    FROM OM_PRODUTO PDT
              INNER JOIN OM_PRODUTO_REPOSITORIO PRP ON PRP.NRPRODUTO_ID = PDT.ID
              INNER JOIN REPOSITORY RPT ON RPT.ID = PRP.NREXTERNAL_REPOSITORY_ID
               LEFT JOIN DEPLOY_REPOSITORY DRP ON DRP.EXTERNAL_REPOSITORY_ID = PRP.ID
                                              AND DRP.DEPLOY_ID IN (:DEPLOY_IDS)
                   WHERE NREXTERNAL_PRODUCT_ID IN (:PRODUCT_IDS)
                     AND RPT.STATUS = 'A'
                     AND PRP.IDATIVO = 'S'";

        $params = array(
            'PRODUCT_IDS' => $productIds,
            'DEPLOY_IDS'  => $deployIds
        );
        $paramTypes = array(
            'PRODUCT_IDS' => Connection::PARAM_STR_ARRAY,
            'DEPLOY_IDS'  => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }

    /**
     * @return array
     */
    public function getMigrationRepositoriesInfo() {
        $query = "SELECT NREXTERNAL_PRODUCT_ID, ID, NAME, 
                         URL, MIG_RELATIVE_PATH, PATH,
                         MAX(INITIAL_MIGRATION_NUMBER) AS INITIAL_MIGRATION_NUMBER
                    FROM (SELECT RPT.NAME, PRP.ID, PDT.NREXTERNAL_PRODUCT_ID,
                                 REPLACE (
                                     REPLACE (
                                         RPT.URL, 
                                         'git@gitlab.teknisa.com:teknisa/entities.git',  'git@192.168.122.56:teknisa/entities.git'
                                     ), 
                                     'git@gitlab.teknisa.com:zeedhi/devops-entities.git',  'git@192.168.122.56:zeedhi/devops_entities.git'
                                 ) AS URL,
                                 PRP.DSDIRRELMIGBD AS MIG_RELATIVE_PATH,
                                 DRP.INITIAL_MIGRATION_NUMBER,
                                 REPLACE(PATH, (SELECT REP.PATH
                                                  FROM REPOSITORY REP
                                                 WHERE REP.PRODUCT = RPT.PRODUCT
                                                   AND REP.STATUS = 'A'
                                                   AND REP.IS_MASTER = 'Y'), '') AS PATH
                            FROM OM_PRODUTO PDT
                      INNER JOIN OM_PRODUTO_REPOSITORIO PRP ON PRP.NRPRODUTO_ID = PDT.ID
                      INNER JOIN REPOSITORY RPT             ON RPT.ID = PRP.NREXTERNAL_REPOSITORY_ID
                       LEFT JOIN DEPLOY_REPOSITORY DRP      ON DRP.EXTERNAL_REPOSITORY_ID = PRP.ID
                           WHERE RPT.STATUS = 'A'
                             AND PRP.IDATIVO = 'S')
                GROUP BY NREXTERNAL_PRODUCT_ID, ID, NAME, URL, MIG_RELATIVE_PATH, PATH";

        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }
}
